"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const CloudWatchLogs = require("aws-sdk/clients/cloudwatchlogs");
const CodeCommit = require("aws-sdk/clients/codecommit");
const common_1 = require("./common");
const cloudwatchlogs = new CloudWatchLogs();
const codecommit = new CodeCommit();
exports.handler = (event = {}) => __awaiter(void 0, void 0, void 0, function* () {
    const requestToken = event.detail['build-id'] + event.detail['build-status'];
    const buildenv = common_1.getBuildEnvironment(event.detail['additional-information'].environment['environment-variables']);
    return new Promise((resolve, reject) => {
        if (!buildenv.CODECOMMIT_PULL_REQUEST_ID) {
            reject("Not a pull request build");
            return;
        }
        resolve(createComment(event.detail, buildenv));
    }).then((comment) => {
        return codecommit.postCommentForPullRequest({
            afterCommitId: buildenv.CODECOMMIT_DESTINATION_COMMIT_ID,
            beforeCommitId: buildenv.CODECOMMIT_SOURCE_COMMIT_ID,
            clientRequestToken: requestToken,
            content: comment,
            pullRequestId: buildenv.CODECOMMIT_PULL_REQUEST_ID,
            repositoryName: buildenv.CODECOMMIT_REPOSITORY_NAME
        }).promise();
    });
});
function createComment(detail, buildenv) {
    return __awaiter(this, void 0, void 0, function* () {
        const buildStatus = detail['build-status'];
        const url = common_1.getBuildJobUrl(detail['build-id']);
        let comment = `# Pull Request Builder\n\n**${buildStatus}**\n\n${url}\n\nSource Commit: ${buildenv.CODECOMMIT_SOURCE_COMMIT_ID}`;
        const logs = detail['additional-information'].logs;
        const artifact = detail['additional-information'].artifact;
        try {
            if (common_1.isFailedBuild(buildStatus) && logs) {
                comment += "\n\nYou must resolve the issue before the branch can be merged.\n\n";
                const rawEvents = yield getLogEvents(logs['group-name'], logs['stream-name']);
                const logLines = rawEvents.events.map(event => event.message).join("");
                comment += "\n```\n" + logLines + "\n```\n";
            }
            if (buildStatus !== 'IN_PROGRESS' && hasS3Artifact(artifact)) {
                comment += "\n\n## Results\n\nArtifact: " + getArtifactUrl(artifact);
            }
        }
        catch (e) {
            comment += `Error creating comment text:\n${e.message}\n\n${e.stack}`;
        }
        return comment;
    });
}
const getLogEvents = (groupName, streamName) => {
    return cloudwatchlogs.getLogEvents({
        logGroupName: groupName,
        logStreamName: streamName,
        limit: 30,
        startFromHead: false
    }).promise();
};
const getS3Artifact = (artifact) => {
    return artifact.location.split(':')[5];
};
const hasS3Artifact = (artifact) => {
    return artifact.location.substr(0, 10) === 'arn:aws:s3';
};
const getArtifactUrl = (artifact) => {
    const s3Artifact = getS3Artifact(artifact);
    return `https://s3.console.aws.amazon.com/s3/buckets/${s3Artifact}/?region=${process.env.AWS_REGION}&tab=overview`;
};
