import { Construct } from "@aws-cdk/core";
import { Project } from "@aws-cdk/aws-codebuild";
import { Repository } from "@aws-cdk/aws-codecommit";
import { Role } from "@aws-cdk/aws-iam";
import { Code, Runtime } from "@aws-cdk/aws-lambda";
export interface BuildFailureEmailProps {
    readonly sourceEmailParam: string;
    readonly notificationEmailParam?: string;
}
export interface CommonFunctionProps {
    readonly code: Code;
    readonly runtime: Runtime;
    readonly role: Role;
}
export interface PullRequestBuilderProps {
    readonly enforceApproval: boolean;
    readonly buildFailureEmailSettings?: BuildFailureEmailProps;
    readonly lambdaRole?: Role;
    readonly project: Project;
    readonly repo: Repository;
}
/**
 * CDK PullRequestBuilder
 *
 * Implements an event-driven workflow using
 * CodeBuild, CodeCommit, CloudWatchEvents,
 * CloudWatchLogs, and Lambda to achieve a basic
 * automated test cycle for pull requests.
 *
 * CodeBuild is started when a pull request is opened
 * or updated. The PR is updated with comments as
 * the build changes its state.
 *
 * Optionally, the job can enforce an approval rule on
 * the PR by configuring PullRequestBuilderProps.enforceApproval
 */
export declare class PullRequestBuilder extends Construct {
    private readonly handlersDir;
    private readonly layersDir;
    private repo;
    private project;
    private serviceRole;
    private commonFunctionProps;
    private isExternalRole;
    private cloudwatchEvents;
    constructor(scope: Construct, id: string, props: PullRequestBuilderProps);
    /**
     * Creates CloudWatch Events on the CodeBuild project
     * and CodeCommit pull requests
     */
    private setCloudWatchEvents;
    /**
     *   Creates a cdk.iam.Role and policies for
     *   the Lambda functions to use.
     *
     *   Requires permissions:
     *   - codebuild:StartBuild
     *   - codecommit:GetCommit
     *   - codecommit:PostCommentOnPullRequest
     *   - logs:GetLogEvents
     *   - ses:SendEmail
     *
     *   If props.enforceApproval is true:
     *   - codecommit:GetPullRequest
     *   - codecommit:UpdatePullRequestApprovalState
     *
     */
    private newServiceRole;
    /**
     * Appends the class'es serviceRole with a passed in policy
     *
     * This no-ops if using an external IAM role for the service.
     */
    private appendServiceRole;
    /**
     * Sets up a CloudWatchEvent to invoke a Lambda function
     * when a pull request is created in a CodeCommit repository.
     * The lambda function will then start a CodeBuild job
     * and supply data about the pull request as environment variables.
     *
     * This function intentionally does not use tryFindChild since each
     * project needs its own function, since there is no way for
     * the Lambda function to know which CodeBuild job to start from the
     * CodeCommit pull request without setting it as an environment
     * variable in the function
     *
     */
    private startBuildOnPullRequestOpen;
    /**
     * Sets up a CloudWatch Event that invokes a Lambda function
     * when a CodeBuild job changes its state. The function
     * will read the build's environment variables to determine
     * the pull request, and then post a comment on it with information
     * regarding the build and its status
     *
     * @param stack? cdk.Stack
     * The stack to check if function already exists
     */
    private postCommentOnBuildStateChange;
    /**
     * Sets up a CloudWatchEvent to invoke a Lambda function
     * when a CodeBuild project suceeds or fails
     *
     * The function will approve or revoke the pull request based
     * on the build status
     *
     * @param stack? cdk.Stack
     * The stack to check if function already exists
     */
    private enforceApprovalOnPullRequest;
    /**
     * When the CodeBuild job fails, email the
     * person who opened the pull request
     *
     * Conditionally CC a configured email address on all failures
     *
     * @param emailSettings? BuildFailureEmailProps
     * @param stack? cdk.Stack
     * The stack to check if function already exists
     */
    private notifyCommitterByEmailOnBuildFailed;
}
